@@ -1047,7 +1047,7 @@ babl_fish_reference_process_double (const Babl *babl,
      if (cmyk_cmyk_no >= cmyk_cmyk_count)
      {
        cmsHPROFILE src_profile = cmsOpenProfileFromMem(source_space->space.icc_profile, source_space->space.icc_length);
-       cmsHPROFILE dst_profile = cmsOpenProfileFromMem(destination_space->space.icc_profile, source_space->space.icc_length);
+       cmsHPROFILE dst_profile = cmsOpenProfileFromMem(destination_space->space.icc_profile, destination_space->space.icc_length);
 
        cmyk_cmyk_source[cmyk_cmyk_no] = source_space;
        cmyk_cmyk_destination[cmyk_cmyk_no] = destination_space;
@@ -1385,8 +1385,6 @@ babl_fish_reference_process (const Babl *babl,
   static const void *type_float = NULL;
   static int allow_float_reference = -1;
 
-  if (!type_float) type_float = babl_type_from_id (BABL_FLOAT);
-
   /* same format in source/destination */
   if (BABL (babl->fish.source) == BABL (babl->fish.destination))
   {
@@ -1422,6 +1420,9 @@ babl_fish_reference_process (const Babl *babl,
     return;
   }
 
+  if (!type_float)
+    type_float = babl_type_from_id (BABL_FLOAT);
+
   if (allow_float_reference == -1)
     allow_float_reference = getenv ("BABL_REFERENCE_NOFLOAT") ? 0 : 1;
 
