@@ -26,8 +26,7 @@
 
 
 int
-main (int    argc,
-      char **argv)
+main (void)
 {
   int OK = 1;
   babl_init ();
