@@ -20,8 +20,7 @@
 #include "babl.h"
 
 int
-main (int    argc,
-      char **argv)
+main (void)
 {
   babl_init ();
   babl_exit ();
